############################################################################
##
## Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
## All rights reserved.
##
## $Id: Predef.py,v 1.8 2002/10/22 11:01:50 sasa Exp $
##
############################################################################

import UIUtils
from snack import *
import sys

from ServiceConfig import ServiceConfig
from ListenerConfig import ListenerConfig
from ProxyConfig import ProxyConfig
from RouterConfig import RouterConfig
from ZUI.Lang import *


predefdb = None
prev_ip = '192.168.1.1'

FormatError = 'error parsing predefined.db'

class PredefinedService:
	def __init__(self):
	        self.id = 'IRC'
		self.service_name = ''
		self.description = ''
		self.serice_port = 0
		self.redir_port = 0
		self.proxy_tmpl = 'PlugProxy'
		self.router_tmpl = 'TransparentRouter'
		self.listen_addr_tmpl = 'SockAddrInet("%(ip)s", %(port)d)'
		self.modules_tmpl = ''
		self.global_tmpl = ''
		self.seq = 0

	def readBlock(self, f):
		block = ''
		line = '#'
		
		while line and (line[0:1] == '#' or line[0:1] == '\n'):
			line = f.readline()

		if line and line[0:1] != '=':
			type = line[0:1]
			if line[1:3] == ' .':
				# multiline block
				line = f.readline()
				while line and line[0:1] != '.':
					block = block + line
					line = f.readline()
			else:
				block = line[2:]
				if block[-1:] == '\n':
				    block = block[:-1]
			return type, block
		return None
		
	def readParse(self, f):
		first = 1
		res = self.readBlock(f)
		while res:
			first = 0
			if res[0] == 'S':
				# service name
				self.service_name = res[1]
			elif res[0] == 'D':
				# description
				self.description = res[1]
			elif res[0] == 'I':
				# unique id
				self.id = res[1]
			elif res[0] == 'P':
				# port number
				self.service_port = int(res[1])
			elif res[0] == 'R':
				# redirected port number
				self.redir_port = int(res[1])
			elif res[0] == 'O':
				# service definition
				self.router_tmpl = res[1]
			elif res[0] == 'X':
				# service definition
				self.proxy_tmpl = res[1]
			elif res[0] == 'L':
				# listener definition
				self.listen_addr_tmpl = res[1]
			elif res[0] == 'M':
				self.modules_tmpl = res[1]
			elif res[0] == 'G':
				self.global_tmpl = res[1]
			else:
				raise FormatError, 'error parsing predefined.db'
			res = self.readBlock(f)
		return not first

	def instantiate(self, instance_config, service_name, ip):

		port = self.redir_port

		service = ServiceConfig(instance_config, service_name, 
				self.modules_tmpl % locals(), 
				self.global_tmpl % locals(),
				self.id)

		service.proxy = ProxyConfig(service, self.proxy_tmpl % locals())
		service.router = RouterConfig(service, self.router_tmpl % locals())
       	
		listener = ListenerConfig(instance_config, self.listen_addr_tmpl % locals())
		listener.service = service
		return service, listener

class PredefDB:
	def __init__(self, filename):
		self.entries = {}
		self.readDb(filename)
		
	def readDb(self, filename):
	        seq = 0
		f = open(filename)
		x = PredefinedService()
		while x.readParse(f):
		        x.seq = seq
			self.entries[x.service_name] = x
			x = PredefinedService()
			seq = seq + 1
			
	def getServiceAndIP(self, instance_config, p):
	    global prev_ip
	    
            screen = UIUtils.screen
            form = GridForm(screen, mylang.get('Add service'), 1, 5)
	    
	    service_name_l = Label(mylang.get('Service name'))
	    service_name = Entry(30, '%s_%s' % (instance_config.name, p.id), returnExit=1)
	    ip_l = Label(mylang.get('IP address to listen on'))
	    ip = Entry(30, prev_ip, returnExit=1)
	    bb = ButtonBar(screen, [(mylang.get("  Ok  "), "ok"), (mylang.get(" Cancel "), "cancel")])
	    
	    form.add(service_name_l, 0, 0, anchorLeft=1)
	    form.add(service_name, 0, 1, padding=(0, 0, 0, 1))
	    form.add(ip_l, 0, 2, anchorLeft=1)
	    form.add(ip, 0, 3, padding=(0, 0, 0, 1))
	    form.add(bb, 0, 4)
	    
	    res = form.runOnce()
	    x = bb.buttonPressed(res)
	    
	    if x == 'ok' or x == None:
	        prev_ip = ip.value()
	        return service_name.value(), ip.value()
	    return None
			
        def instantiate(self, instance_config, p):
	    
	    try:
	    	service_name, ip = self.getServiceAndIP(instance_config, p)
	    except TypeError:
	        return
		
	    if instance_config.services.has_key(service_name):
	        UIUtils.MessageForm(title=mylang.get("Message"), text=mylang.get("This service (%s) already exists.") % service_name).run()
	        return
	    service, listener = p.instantiate(instance_config, service_name, ip)
            
            instance_config.services[service.name] = service
            instance_config.services_lb.append(service.name, service)
            instance_config.listeners.append(listener)
            instance_config.listeners_lb.append(listener.addr[-20:], listener)
	    instance_config.listeners_lb.setCurrent(listener)
			
	def runForm(self, instance_config):
            screen = UIUtils.screen
            form = GridForm(screen, mylang.get('Select predefined settings to use'), 2, 1)
            
            leftgrid = Grid(1, 2)
            
            predef_l = Label(mylang.get('Predefined module'))
            predef = Listbox(width=50, height=8, scroll=1, border=1, returnExit=1)
            
            protocols = []
            for p in self.entries.keys():
                protocols.append(p)
                
            protocols.sort()
            for p in protocols:
                predef.append(p, self.entries[p])
                
            leftgrid.setField(predef_l, 0, 0, anchorLeft=1)
            leftgrid.setField(predef, 0, 1)
            
            main_bb = ButtonBar(screen, [(mylang.get("  Ok  "), "ok"), (mylang.get("Cancel"), "cancel"), (mylang.get(" Help "), "help")], horizontal=0, compact=1)
            
            form.add(leftgrid, 0, 0)
            form.add(main_bb, 1, 0)
            
	    quit = 0
	    while not quit:
  	        res = form.run()
	        x = main_bb.buttonPressed(res)

            	try:
	            p = None
                    p = predef.current()
            	except KeyError:
                    pass
            	if x == "ok" or x == None:
		    x = "ok"
		    quit = 1
                elif x == "help":
	            UIUtils.HelpForm("Help", p.description).run()
		else:
		    quit = 1
	    screen.popWindow()
            if x == "ok" and p:
                self.instantiate(instance_config, p)
		
