############################################################################
##
## Copyright (c) 2000-2001 BalaBit IT Ltd, Budapest, Hungary
## All rights reserved.
##
## $Id: ServiceConfig.py,v 1.7 2002/10/22 11:01:50 sasa Exp $
##
############################################################################

import UIUtils
from Config import Config
from RouterConfig import RouterConfig
from ProxyConfig import ProxyConfig
from snack import *
from ZUI.Lang import *

class ServiceConfig(Config):
    def __init__(self, owner, name, import_script='', global_script='', global_tag=''):
        self.owner = owner
        self.name = name
        self.router = RouterConfig(self, 'TransparentRouter')
        self.proxy = ProxyConfig(self, 'PlugProxy')
	self.global_script = global_script
	self.import_script = import_script
	self.global_tag = global_tag

    def importHook(self, output, gi):

        for x in self.router, self.proxy:
            if x:
                x.importHook(output, gi)
	self.generateImports(self.import_script, output, gi)

    def globalHook(self, output, gi):
        self.generateGlobal(self.global_script, self.global_tag, output, gi)

    def generateConfig(self, output, gi):
        info = {
            "name": self.name
            }                       
        output.write('\t' * gi.indent_level + ('Service("%(name)s", ' % info))
        self.proxy.generateConfig(output, gi)
        output.write(', router=')
        self.router.generateConfig(output, gi)
        output.write(')\n')

    def addZone(self, direction):
        screen = UIUtils.screen

        form = GridForm(screen, mylang.get("Add zone"), 2, 2)

        zones_l = Label(mylang.get("Available zones"))
        zones = Listbox(height=5, width=20, border=1, scroll=1, returnExit=1)
        main_bb = ButtonBar(screen, [(mylang.get("  Ok  "), "ok"), (mylang.get("Cancel"), "cancel")], horizontal=0, compact=1)

        for z in self.owner.owner.zones.keys():
            zones.append(z, self.owner.owner.zones[z])

        form.add(zones_l, 0, 0)
        form.add(zones, 0, 1)
        form.add(Label(""), 1, 0)
        form.add(main_bb, 1, 1)

        res = form.runOnce()
        main_action = main_bb.buttonPressed(res)

        if main_action == "ok" or main_action == None:
            try:
                z = zones.current()
                getattr(self, direction).append(z.name, z)
                getattr(z, direction + '_services')[self.name] = 1
            except KeyError:
                pass
        
    def editScript(self, which):
        x = getattr(self, which + '_script')
	UIUtils.HelpForm(mylang.get('Script'), x).run()

    def runForm(self):
        screen = UIUtils.screen

	form = GridForm(screen, mylang.get("Edit service %s") % self.name, 2, 1)
	
	proxy_l = Label(mylang.get("Proxy"))
	proxy = self.proxy.addForm()
	router_l = Label(mylang.get("Router"))
	router = self.router.addForm()

	data_bb = ButtonBar(screen, [(mylang.get("Import"), "import"), (mylang.get("Globals"), "globals")], compact=1, horizontal=1)
	
	lowgrid = Grid(4, 2)
	outbound_l = Label(mylang.get("Source Zones"))
	self.outbound = outbound = Listbox(width=20, height=5, border=1, scroll=1)
	outbound_bb = ButtonBar(screen, [(mylang.get(" Add  "), "add"), (mylang.get("Delete"), "delete")], compact=1, horizontal=0)
				
	inbound_l = Label(mylang.get("Destination Zones"))
	self.inbound = inbound = Listbox(width=20, height=5, border=1, scroll=1)
	inbound_bb = ButtonBar(screen, [(mylang.get(" Add  "), "add"), (mylang.get("Delete"), "delete")], compact=1, horizontal=0)
	

	for z in self.owner.owner.zones.keys():
		if self.owner.owner.zones[z].inbound_services.has_key(self.name):
			inbound.append(z, self.owner.owner.zones[z])
		if self.owner.owner.zones[z].outbound_services.has_key(self.name):
			outbound.append(z, self.owner.owner.zones[z])
		
	lowgrid.setField(outbound_l, 0, 0)
	lowgrid.setField(outbound, 0, 1)
	lowgrid.setField(Label(""), 1, 0)
	lowgrid.setField(outbound_bb, 1, 1)

	lowgrid.setField(inbound_l, 2, 0)
	lowgrid.setField(inbound, 2, 1)
	lowgrid.setField(Label(""), 3, 0)
	lowgrid.setField(inbound_bb, 3, 1)
	
	leftgrid = Grid(1, 6)
	leftgrid.setField(proxy_l, 0, 0, anchorLeft=1)
	leftgrid.setField(proxy, 0, 1, padding=(0, 0, 0, 1), anchorLeft=1, anchorRight=1)
	leftgrid.setField(router_l, 0, 2, anchorLeft=1)
	leftgrid.setField(router, 0, 3, anchorLeft=1, anchorRight=1)
	leftgrid.setField(lowgrid, 0, 4, padding=(0, 1, 0, 0))
	leftgrid.setField(data_bb, 0, 5, anchorLeft=1, padding=(0, 1, 0, 0))
	
	main_bb = ButtonBar(screen, [(mylang.get("Close"), "close")], horizontal=0, compact=0)

	form.add(leftgrid, 0, 0)
	form.add(main_bb, 1, 0)
	
	quit = 0
	while not quit:
	    res = form.run()
	    main_action = main_bb.buttonPressed(res)
	    inbound_action = inbound_bb.buttonPressed(res)
	    outbound_action = outbound_bb.buttonPressed(res)
	    data_action = data_bb.buttonPressed(res)
	    
	    if self.proxy.runForm(res):
	        pass
	    elif self.router.runForm(res):
	        pass
	    elif data_action == "import":
	    	self.editScript('import')
	    elif data_action == "globals":
	        self.editScript('global')
	    elif inbound_action == "add":
	        self.addZone('inbound')
	    elif inbound_action == "delete":
	        try:
	            x = inbound.current()
	            inbound.delete(x)
	            del x.inbound_services[self.name]
	        except KeyError:
	            pass
            elif outbound_action == "add":
	        self.addZone('outbound')
            elif outbound_action == "delete":
	        try:
	            x = outbound.current()
	            outbound.delete(x)
	            del x.outbound_services[self.name]
	        except KeyError:
	            pass
	    elif main_action == "close" or main_action == None:
	        # ok or enter
                #self.proxy = proxy.value()
	        #self.router = router.value()
                quit = 1
	
	del self.inbound
	del self.outbound
	self.proxy.endForm()
	self.router.endForm()
	screen.popWindow()

